{% extends "accounts/base.html" %}
{% load djblets_deco i18n %}

{% block title %}{% trans "Password Reset Successful" %}{% endblock %}

{% block auth_content %}
<div class="auth-header">
 <h1>{% trans "Password Reset Successful" %}</h1>
</div>

<p>
{%  blocktrans %}
 Check your e-mail. If all went well, you should find instructions on
 resetting your password.
{%  endblocktrans %}
</p>
<p>
{%  blocktrans %}
 (You can probably close this tab now.)
{%  endblocktrans %}
</p>
{% endblock auth_content %}
