{% extends "accounts/base.html" %}
{% load djblets_deco i18n %}

{% block title %}{% trans "Recover Password" %}{% endblock %}

{% block auth_content %}
<div class="auth-header">
 <h1>{% trans "Recover Password" %}</h1>
 <p>
{%  blocktrans %}
  Forget your password? That's okay, just enter your e-mail
  address and we'll get you going again.
{%  endblocktrans %}</p>
 </p>
{%  if form.errors %}
{%   errorbox %}{{form.non_field_errors}}{% enderrorbox %}
{%  endif %}
</div>

<form method="post" action="." class="auth-section main-auth-section">
 {% csrf_token %}

 <div class="auth-form-row auth-field-row">
  <label for="id_email">{% trans "E-mail address" %}:</label>
  {{form.email}}
  {{form.errors.email}}
 </div>

 <div class="auth-form-row">
  <div class="auth-button-container">
   <input type="submit" class="primary" value="{% trans "Recover" %}" />
  </div>
 </div>
</form>
{% endblock auth_content %}
