form td {
  padding: 8px 8px 4px 4px;
}
form td.label {
  text-align: right;
  font-weight: bold;
}
form th {
  text-align: right;
  vertical-align: top;
}
label.required:after {
  content: ' *';
  font-weight: bold;
  font-size: 10px;
  color: darkred;
}
td.help {
  font-size: 10px;
  color: #333;
  padding-top: 0;
}
.errorlist {
  color: #DD0000;
  display: inline;
  font-weight: bold;
  margin: 0 0 0 15px;
  padding: 4px 8px;
}
.errorlist li {
  display: inline;
}
