#custom_policy_editor {
  margin: 0;
  width: 65em;

  p {
    font-size: 110%;
    margin: 1em;
  }

  .CodeMirror {
    border-top: 1px #DDD solid;
    border-bottom: 1px #DDD solid;
  }
}

.config-forms-container .config-forms-page-content {
  .config-site-api-tokens {
    .config-api-token-note {
      color: #888;

      &.empty {
        color: #CCC;
        font-style: italic;
      }
    }

    .config-api-token-value {
      font-family: monospace;
      font-size: 120%;
    }

    .config-forms-list-action-label-policy-custom {
      color: blue;
      cursor: pointer;
      text-decoration: underline;
    }

    .config-forms-list-action-policy {
      text-align: right;
      width: 6.5em;
    }

    .generate-api-token {
      background: white;

      a {
        color: blue;
        font-size: 90%;
        text-decoration: none;
      }
    }

    .rb-icon {
      vertical-align: middle;
    }
  }
}
