"""Extension unit test helpers.

This module provides handy imports for extension testing classes:

* :py:class:`~reviewboard.extensions.testing.testcases.ExtensionTestCase`
"""

from __future__ import unicode_literals

from reviewboard.extensions.testing.testcases import ExtensionTestCase


__all__ = [
    'ExtensionTestCase',
]

__autodoc_excludes__ = __all__
