from __future__ import unicode_literals

from reviewboard.scmtools.tests.testcases import SCMTestCase


__all__ = [
    # Backwards-compatibility for third-party modules that used this import.
    'SCMTestCase',
]
