from __future__ import unicode_literals

from reviewboard.hostingsvcs.tests.testcases import ServiceTests


__all__ = [
    # Backwards-compatibility for third-party modules that used this import.
    'ServiceTests',
]
