@import (reference) "../defs.less";

.upload-diff-ui() {
  #prompt-for-basedir,
  #prompt-for-diff,
  #prompt-for-parent-diff,
  #prompt-for-change-number,
  #processing-diff,
  #uploading-diffs,
  #error-indicator {
    display: none;
  }

  #processing-diff,
  #uploading-diffs {
    .spinner {
      width: 16px;
      height: 16px;
      font-size: 16px;
      margin: auto;
    }
  }

  #error-contents,
  #parent-diff-error-contents {
    line-height: 12pt;
    margin-bottom: 1em;
    position: relative;

    img {
      margin-right: 5px;
      vertical-align: sub;
    }
  }

  .input {
    .border-radius(@box-border-radius);
    background-color: lighten(@light-grey, 10%);
    position: relative;
    text-align: center;
    vertical-align: middle;

    .startover {
      position: absolute;
      bottom: 0;
      right: 5px;
      line-height: 14pt;
    }
  }

  .drag-hover {
    border: 1px solid darken(@grey, 4%);
    .box-shadow(inset @box-shadow);
  }
}
